import { FormDefFixedVar } from "../form-def-fixedvar";
import { CalculatorType, ComputeNodeType, IObservable, Observer } from "jalhyd";
import { FormResultFixedVar } from "../form-result-fixedvar";
import { FormulaireDefinition } from "../form-definition";
import { FormDefSection } from "../form-def-section";
import { CalculatorResults } from "../../../results/calculator-results";
import { FormDefParamToCalculate } from "../form-def-paramcalc";
import { FieldSet } from "../../fieldset";
import { FormComputeFixedVar } from "../form-compute-fixedvar";

export class FormulaireRegimeUniforme extends FormulaireDefinition implements Observer {
    private _formFixedVar: FormDefFixedVar;

    private _formParamCalc: FormDefParamToCalculate;

    private _formSection: FormDefSection;

    private _formCompute: FormComputeFixedVar;

    private _formResult: FormResultFixedVar;

    constructor() {
        super(CalculatorType.RegimeUniforme)
        this._formFixedVar = new FormDefFixedVar(this);
        this._formParamCalc = new FormDefParamToCalculate(this);
        this._formSection = new FormDefSection(this);
        this._formResult = new FormResultFixedVar(this, true);
        this._formCompute = new FormComputeFixedVar(this, this._formResult);
    }

    protected initParse() {
        this._formParamCalc.initParse();
    }

    protected parseOptions(json: {}) {
        this._formSection.parseOptions(json);
    }

    public afterParseFieldset(fs: FieldSet) {
        this._formSection.afterParseFieldset(fs);
    }

    protected completeParse(json: {}) {
        this._formParamCalc.parseOptions(json);
        super.completeParse(json);
    }

    /**
     * gestion du clic sur les radios "paramètre fixé, à varier, à calculer"    
     */
    public onRadioClick(info: string) {
        super.onRadioClick(info);
        this._formParamCalc.onRadioClick(info);
    }

    public resetResults() {
        this._formResult.resetResults();
    }

    public doCompute() {
        this._formCompute.doCompute();
    }

    public get hasResults(): boolean {
        return this._formResult.hasResults;
    }

    public get results(): CalculatorResults[] {
        return this._formResult.results;
    }

    public reset() {
        super.reset();
        this._formParamCalc.parseOptions(this.jsonConfig);
    }

    // interface Observer

    update(sender: IObservable, data: any) {
        // changement de propriété du FieldSet contenant le select de choix du type de section
        if (sender instanceof FieldSet && sender.id === "fs_section" && data.action == "propertyChange") {
            this.replaceCurrentSessionNub(sender.properties);
            for (const fs of this.allFieldsets)
                fs.setSessionNub(this._currentSessionNub);
            this.reset();
        }
    }
}
