import { StructureType, LoiDebit, ComputeNodeType, MethodeResolution } from "jalhyd";

import { Field } from "./field";
import { SelectEntry } from "./select-entry";
import { Dependency } from "./dependency/dependency";
import { DependencyConditionType } from "./dependency/dependency-condition";
import { ValueDependencyCondition } from "./dependency/value-dependency-condition";
import { StringMap } from "../stringmap";
import { FormulaireNode } from "./formulaire-node";

export class SelectField extends Field {
    private _entries: SelectEntry[];

    private _selectedEntry: SelectEntry;

    constructor(parent: FormulaireNode) {
        super(parent);
        this._entries = [];
    }

    public get entries() {
        return this._entries;
    }

    public addEntry(e: SelectEntry) {
        this._entries.push(e);
        if (this._selectedEntry == undefined)
            this._selectedEntry = e;
    }

    public getSelectedEntryFromValue(val: any): SelectEntry {
        for (const se of this._entries)
            if (se.value === val)
                return se;
        return undefined;
    }

    public getValue() {
        return this._selectedEntry;
    }

    public setValue(v: SelectEntry) {
        if (this._selectedEntry !== v) {
            this._selectedEntry = v;
            this.notifyObservers({
                "action": "select",
                "value": v
            }, this);
        }
    }

    public get isValid(): boolean {
        return true;
    }

    public getLabel() {
        if (this._selectedEntry == undefined)
            return undefined;
        return this._selectedEntry.label;
    }

    public verifiesDependency(d: Dependency): boolean {
        if (super.verifiesDependency(d))
            return true;

        switch (d.masterCondition.type) {
            case DependencyConditionType.HasValue:
                let mc: ValueDependencyCondition = <ValueDependencyCondition>d.masterCondition;
                return this._selectedEntry.id === mc.value;

            default:
                throw "SelectField.verifiesDependency() : type de condition '" + DependencyConditionType[d.masterCondition.type] + "' non pris en charge";
        }
    }

    public updateLocalisation(loc: StringMap) {
        super.updateLocalisation(loc);

        for (let e of this._entries) {
            e.label = loc[e.id];
        }
    }

    public parseConfig(field: {}, data?: {}) {
        this._confId = field["id"];
        const values = field["select"];
        for (const v of values) {
            let val;
            if (v.enum) {
                const tmp = v.enum.split(".");
                switch (tmp[0]) {
                    case "ComputeNodeType":
                        val = ComputeNodeType[tmp[1]];
                        break;

                    case "StructureType":
                        val = StructureType[tmp[1]];
                        break;

                    case "LoiDebit":
                        val = LoiDebit[tmp[1]];
                        break;

                    case "MethodeResolution":
                        val = MethodeResolution[tmp[1]];
                        break;

                    default:
                        throw new Error(`type d'enum ${tmp[0]} non pris en charge`);
                }
            }
            else if (v.value)
                val = v.value;

            const id = v["id"];
            const e: SelectEntry = new SelectEntry(id, val);
            this.addEntry(e);
        }
    }
}
